/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.io.*;
import java.util.*; //pour StringTokenizer, Hashtable...
import java.awt.*;
import javax.swing.*;

public class AnatoliaAreFileReader extends AnatoliaAreReader
{
  public AnatoliaAreFileReader(AreaMap areaMap)
  {
    super(areaMap);
  }

  public void read(BufferedReader buffer) {}

  public void load(String fileName) throws IOException
  {
    int progressIndex = 0;
    AnatoliaAreReader reader = null;
    ProgressMonitor progressMonitor = new ProgressMonitor(areaMap.mainScreen, "Loading","",0,18);

      progressMonitor.setProgress(progressIndex++);
      BufferedReader input = new BufferedReader(new FileReader(fileName));
      String nextLine = super.trim(input);
      while(!nextLine.startsWith("#$"))
      {
        if (nextLine.startsWith("#AREA"))
        {
          progressMonitor.setNote("Loading #AREA...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaAreaHeaderReader(areaMap);
          reader.read(input);
        }
        if (nextLine.startsWith("#AREADATA"))
        {
          progressMonitor.setNote("Loading #AREADATA...");
          progressMonitor.setProgress(progressIndex++);
        }
        else if (nextLine.startsWith("#FLAG"))
        {
          progressMonitor.setNote("Loading #FLAG...");
          progressMonitor.setProgress(progressIndex++);
          nextLine = super.trim(input);
        }
        if (nextLine.startsWith("#HELPS")) //not taken in account (I have no doc)
        {
          progressMonitor.setNote("Loading #HELPS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaHelpsReader(areaMap);
          reader.read(input);
        }
        if (nextLine.startsWith("#RESETMESSAGE"))
        {
          progressMonitor.setNote("Loading #RESETMESSAGE...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaResetMessageReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#MOBILES"))
        {
          progressMonitor.setNote("Loading #MOBILES...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaMobilesReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#MOBOLD"))
        {
          progressMonitor.setNote("Loading #MOBOLD...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaOldMobilesReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#OBJECTS"))
        {
          progressMonitor.setNote("Loading #OBJECTS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaObjectsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#OBJOLD"))
        {
          progressMonitor.setNote("Loading #OBJOLD...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaOldObjectsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#ROOMS"))
        {
          progressMonitor.setNote("Loading #ROOMS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaRoomsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#RESETS"))
        {
          progressMonitor.setNote("Loading #RESETS...");
          progressMonitor.setProgress(progressIndex++);
          connectRooms();
          progressMonitor.setNote("Connecting the rooms...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaResetsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#SPECIALS"))
        {
          progressMonitor.setNote("Loading #SPECIALS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaSpecialsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#SHOPS"))
        {
          progressMonitor.setNote("Loading #SHOPS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaShopsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#OMPROGS"))
        {
          progressMonitor.setNote("Loading #OMPROGS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaOMProgsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#OLIMITS"))
        {
          progressMonitor.setNote("Loading #OLIMITS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaOLimitsReader(areaMap);
          reader.read(input);
        }
        else if (nextLine.startsWith("#PRACTICERS"))
        {
          progressMonitor.setNote("Loading #PRACTICERS...");
          progressMonitor.setProgress(progressIndex++);
          reader = new AnatoliaPracticersReader(areaMap);
          reader.read(input);
        }
        nextLine = super.trim(input);
      }

      progressMonitor.setNote("Converting old formats...");
      progressMonitor.setProgress(progressIndex++);
      convertOldMobiles();
      convertOldObjects();

      progressMonitor.setNote("Loading complete");
      progressMonitor.setProgress(progressIndex++);
      progressMonitor.close();
      input.close();
  }

  public void connectRooms()
  {
    AreaMap tempAreaMap = new AreaMap(null);
    RoomCollection outsideRooms = new RoomCollection(tempAreaMap);
    Iterator it = areaMap.theRooms.anatoliaCollection.getIterator();
    int k=0;
    int step = 2*Room.SIZE;
    while(it.hasNext())
    {
      ++k;
      AnatoliaRoom anatoliaRoom = (AnatoliaRoom) it.next();
      Room room = anatoliaRoom.myRoom;
      for(int i=0 ; i<6 ; ++i)
      {
        Pair pair = room.connectingToRooms[i];
        if (pair == null) continue;
        int vnum = 0;
        try{vnum = ((Integer) pair.getFirst()).intValue();}
        catch(Exception e) {continue;}
        AnatoliaRoom toAnatoliaRoom = (AnatoliaRoom) areaMap.theRooms.anatoliaCollection.getAnatoliaThingOfVnum(vnum);
        Room toRoom = (toAnatoliaRoom == null) ? null : toAnatoliaRoom.myRoom;
        if (toRoom == null) //this was an outside room
        {
          toAnatoliaRoom = (AnatoliaRoom) outsideRooms.anatoliaCollection.getAnatoliaThingOfVnum(vnum);
          toRoom = (toAnatoliaRoom == null) ? null : toAnatoliaRoom.myRoom;
          if (toRoom == null)
          {
            toRoom = new Room(areaMap, areaMap.theRooms.anatoliaCollection);
            toRoom.anatoliaRoom.imposeVnum(vnum);
            toRoom.anatoliaRoom.setExtern(true);
            outsideRooms.add(toRoom);
          }
        }
        Wire wire = new Wire();
        byte toDoor = toRoom.lookForDoorConnectingToRoom(room);
        wire.createConnect(room, (byte) i, toRoom, toDoor);
        room.setLocation((k*step)%(10*step), (k/10)*step);
        areaMap.theWires.add(wire);
      }// end for each direction
    }//end while room
    //add the created outside rooms
    it = outsideRooms.getIterator();
    while(it.hasNext())
    {
      areaMap.theRooms.add((Room) it.next());
    }

    //room placement
    LinkedList roomsToPlace = new LinkedList();
    Set placedRooms  = new HashSet();
    Set doneRooms  = new HashSet();
    it = areaMap.theRooms.getIterator();
    //fills the rooms to place
    while(it.hasNext()) roomsToPlace.add(it.next());

    int deltaX=0;
    int deltaY=0;
    int currentX=0;
    int currentY=0;
    int minX = Integer.MAX_VALUE;
    int minY = Integer.MAX_VALUE;
    //place the rooms
    Pair pair = null;
    it = roomsToPlace.iterator();
    while(!roomsToPlace.isEmpty())
    {
      Room room = (Room) roomsToPlace.removeFirst();
      if (!doneRooms.contains(room))
      {
        doneRooms.add(room);
        if (!placedRooms.contains(room))
        {
          room.setLocation(currentX+(Room.SIZE*3)/2,currentY+(Room.SIZE*3)/2);
          placedRooms.add(room);
        }
        for(int i=0 ; i<6 ; ++i)
        {
          pair = room.connectingToRooms[i];
          if (pair != null)
          {
            Room toRoom = (Room) pair.getFirst();
            if (!doneRooms.contains(toRoom))
            {
              roomsToPlace.remove(toRoom);
              roomsToPlace.addFirst(toRoom);
              placedRooms.add(toRoom);
              toRoom.layer = room.layer;
              switch(i)
              {
                case AnatoliaConsts.NORTH : deltaX = 0; deltaY = -2*Room.SIZE; break;
                case AnatoliaConsts.EAST  : deltaX = +2*Room.SIZE; deltaY = 0; break;
                case AnatoliaConsts.SOUTH : deltaX = 0; deltaY = +2*Room.SIZE; break;
                case AnatoliaConsts.WEST  : deltaX = -2*Room.SIZE; deltaY = 0; break;
                case AnatoliaConsts.UP    : deltaX = -2*Room.SIZE; deltaY = -2*Room.SIZE;
                                           ++toRoom.layer;
                                           break;
                case AnatoliaConsts.DOWN  : deltaX = +2*Room.SIZE; deltaY = +2*Room.SIZE;
                                           --toRoom.layer;
                                           break;
                default: deltaX=0;deltaY=0;break;
              }
              areaMap.selectedLayers.add(new Integer(room.layer));
              areaMap.selectedLayers.add(new Integer(toRoom.layer));
              currentX = room.getX()+deltaX;
              currentY = room.getY()+deltaY;
              toRoom.setLocation(currentX, currentY);
              if (currentX < minX) minX = currentX;
              if (currentY < minY) minY = currentY;
            }//end if (!doneRooms.contains(toRoom))
          }//end pair !=null
        }//end each direction
      }//end doneRooms.contains
    }//end while room

    //translate the rooms for them to have positive coords
    minX = Math.min(minX,0);
    minY = Math.min(minY,0);
    it = areaMap.theRooms.getIterator();
    while(it.hasNext())
    {
      Room room = (Room) it.next();
      room.setLocation(room.getX()-minX,room.getY()-minY);
    }
  }//end function

  public void convertOldMobiles()
  {
    Iterator it = this.areaMap.theMobiles.getIterator();
    while(it.hasNext())
    {
      AnatoliaMobile mobile = (AnatoliaMobile) it.next();
      convertOldMobile(mobile);
    }
  }

  public void convertOldObjects()
  {
    Iterator it = this.areaMap.theObjects.getIterator();
    while(it.hasNext())
    {
      AnatoliaObject object = (AnatoliaObject) it.next();
      convertOldObject(object);
    }
  }

  public void convertOldMobile(AnatoliaMobile mobile)
  {
    //convert an OLD format mob to a new mob
    String codeWarrior = AnatoliaConsts.mobileActFlagTexts[AnatoliaConsts.MOBILE_ACT_FLAG_WARRIOR].getCode();
    if (mobile.actFlags.indexOf(codeWarrior) < 0)
    {
      mobile.actFlags += codeWarrior;
    }

    int type   = mobile.level*mobile.level*27/40;
    int number = Math.min(type/40+1,10);
    type = Math.max(2,type/number);
    int bonus = (int) Math.max(0,mobile.level*(8+mobile.level)*.9-number*type);
    mobile.hitDice  = new DiceRoll(number,type,bonus);
    mobile.manaDice = new DiceRoll(mobile.level, 10,100);

    type = mobile.level*7/4;
    number = Math.min(type/8+1,5);
    type = Math.max(2,type/number);
    bonus = Math.max(0,mobile.level*9/4-number*type);
    mobile.damageDice = new DiceRoll(number,type,bonus);

    mobile.damageType = AnatoliaConsts.DAMAGE_TYPE_SLASH;
    mobile.damage     = AnatoliaConsts.DAMAGE_TYPE_SLASH_SLASH;

    mobile.pierceAC  = 100+(mobile.level*(-200))/32;
    mobile.bashAC    = 100+(mobile.level*(-200))/32;
    mobile.slashAC   = 100+(mobile.level*(-200))/32;
    mobile.magicAC   = 100+(mobile.level*(-100))/32;

    mobile.size = AnatoliaConsts.SIZE_MEDIUM;
    mobile.treasure /= 100;
    mobile.material = "";
  }//end function convertOldMobile

  public void convertOldObject(AnatoliaObject object)
  {
    //object conversion from OLD format to new format
    int level = 0;
    Iterator it = object.instances.iterator();
    while(it.hasNext())
    {
      InstanceOfMobile mobile    = null;
      InstanceOfObject container = null;
      InstanceOfObject instance = (InstanceOfObject) it.next();
      try {mobile = (InstanceOfMobile) instance.getParent();}
      catch(Exception e) {}
      try {container = (InstanceOfObject) instance.getParent();}
      catch(Exception e) {}

      if (mobile != null)
      {
        if (mobile.anatoliaMobile.isShopKeeper)
        {
          level = Math.max(level,mobile.anatoliaMobile.level-2);
        }
        else
        {
          level = Math.max(level,mobile.anatoliaMobile.level);
        }
      }
      else if (container != null)
      {
        level = Math.max(level,container.anatoliaObject.level);
      }
    }

    object.level = (byte) level;
    object.cost = 10*object.level;
    int temp;
    switch(object.type)
    {
      case AnatoliaConsts.OBJECT_TYPE_STAFF : object.v2 = object.v1;
                                             break;
      case AnatoliaConsts.OBJECT_TYPE_WEAPON :
             temp = Math.min(object.level/4+1,5);
             object.v1 = Integer.toString(temp);
             object.v2 = Integer.toString((object.level+7)/temp);
             break;
      case AnatoliaConsts.OBJECT_TYPE_ARMOR :
             object.v0 = Integer.toString(object.level/5+3);
             object.v1 = object.v0;
             object.v2 = object.v0;
             break;
      case AnatoliaConsts.OBJECT_TYPE_MONEY :
             object.v0 = Integer.toString(object.cost);
             break;
      default : break;
    }
  }//end function convertOldObject

}//endclass