/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*; //for HashMap and LinkedList
import java.io.*;   //for archivation

public class AnatoliaMobile extends AnatoliaThing
{
   LinkedList instances = new LinkedList(); //list of InstanceOfMobile of this mobile
   private Map nbInstancesPerRoom = new HashMap(); //<AnatoliaRoom, nbInstances in this room>

   String nameList = "a mobile";
   String shortDescription = "this mobile is boring";
   String longDescription = "this mobile is very boring";
   String lookDescription = "this mobile is very very boring";
   int race = AnatoliaConsts.RACE_TYPE_HUMAN;
   String actFlags = ""+AnatoliaConsts.mobileActFlagTexts[AnatoliaConsts.MOBILE_ACT_FLAG_IS_NPC].getCode();
   String affFlags = "";
   String offFlags = "";
   int alignment = 0;
   int mobileGroup = 0;
   int level = 1;
   int hitBonus = 0;
   DiceRoll hitDice = new DiceRoll();
   DiceRoll manaDice = new DiceRoll();
   DiceRoll damageDice = new DiceRoll();
   byte damageType = AnatoliaConsts.DAMAGE_TYPE_BASH;
   byte damage = AnatoliaConsts.DAMAGE_TYPE_BASH_PUNCH;
   int pierceAC = 0;
   int bashAC = 0;
   int slashAC = 0;
   int magicAC = 0;
   String immFlags = "";
   String resFlags = "";
   String vulnFlags = "";
   String detectFlags = "";
   byte startPosition   = AnatoliaConsts.POSITION_STANDING;
   byte defaultPosition = AnatoliaConsts.POSITION_STANDING;
   byte sex = AnatoliaConsts.SEX_MALE;
   int treasure = 0;
   String corpseFlags = "";
   String formFlags = "";
   String partsFlags = "";
   byte size = AnatoliaConsts.SIZE_LARGE;
   String material = "";
   String special = "";

   //shopkeeper
   boolean isShopKeeper = false;
   int nbItemSold = 0;
   int[] itemSold = new int[5];
   int profitBuy = 100;
   int profitSell = 100;
   byte openHour = 0;
   byte closeHour = 23;

   //special prog
   String[] specialProgs = new String[AnatoliaConsts.mobileProgTexts.length]; //list of String for special prog

   //practicer
   boolean[] practicer = new boolean[AnatoliaConsts.mobilePracticerTexts.length];

   public AnatoliaMobile(AnatoliaCollection anatoliaCollection)
   {
      super(anatoliaCollection);
      for(int i=0 ; i<specialProgs.length ; ++i)
        specialProgs[i] = "";
      for(int i=0 ; i<practicer.length ; ++i)
        practicer[i] = false;
   }

   public Set getSetOfNbInstancesPerRoom() {return nbInstancesPerRoom.entrySet();}
   public void resetInstances()
   {
      instances.clear();
      nbInstancesPerRoom = new HashMap();
   }

   public void clear()
   {
      nbInstancesPerRoom.clear();
      for(int i=0 ; i<specialProgs.length ; ++i)
        specialProgs[i] = "";
      for(int i=0 ; i<practicer.length ; ++i)
        practicer[i] = false;
   }

   public void addInstance(InstanceOfMobile instance, AnatoliaRoom room)
   {
      instances.add(instance);
      Integer nbInstances = (Integer) nbInstancesPerRoom.get(room);
      if (nbInstances == null)
      {
         nbInstances = new Integer(1);
      }
      else
      {
         nbInstances = new Integer(nbInstances.intValue()+1);
      }
      nbInstancesPerRoom.put(room, nbInstances);
   }

   public void removeInstance(InstanceOfMobile instance)
   {
      AnatoliaRoom room = AreaBuilderConsts.getRoomNode(instance.getTheParent());
      instances.remove(instance);
      Integer nbInstances = (Integer) nbInstancesPerRoom.get(room);
      if (nbInstances != null)
      {
         nbInstances = new Integer(nbInstances.intValue()-1);
         if (nbInstances.intValue() > 0)
         {
            nbInstancesPerRoom.put(room, nbInstances);
         }
         else
         {
            nbInstancesPerRoom.remove(room);
         }
      }
   }

   public int getTotalInstances() {return instances.size();}
   public int getNbInstancesInRoom(AnatoliaRoom room)
   {
      Integer number = (Integer) nbInstancesPerRoom.get(room);
      if (number == null)
        return 0;
      else
        return number.intValue();
   }

   public void writeObject(ObjectOutputStream output) throws IOException
   {
      super.writeObject(output);

      //instances
      Iterator it = instances.iterator();
      output.writeInt(instances.size());
      while(it.hasNext())
      {
         output.writeLong(((InstanceOfMobile) it.next()).getObjectID());
      }

      //nb instances per room
      Set entrySet = nbInstancesPerRoom.entrySet();
      output.writeInt(entrySet.size());
      it = entrySet.iterator();
      Map.Entry entry = null;
      while(it.hasNext())
      {
         entry = (Map.Entry) (it.next());
         output.writeLong(((AnatoliaRoom) entry.getKey()).myRoom.getObjectID()); //roomID
         output.writeInt(((Integer) entry.getValue()).intValue());              //nb instances
      }

      //datas
      output.writeInt(super.getVnum());
      output.writeUTF(nameList);
      output.writeUTF(shortDescription);
      output.writeUTF(longDescription);
      output.writeUTF(lookDescription);
      output.writeInt(race);
      output.writeUTF(actFlags);
      output.writeUTF(affFlags);
      output.writeUTF(offFlags);
      output.writeInt(alignment);
      output.writeInt(mobileGroup);
      output.writeInt(level);
      output.writeInt(hitBonus);
      hitDice.writeObject(output);
      manaDice.writeObject(output);
      damageDice.writeObject(output);
      output.writeByte(damageType);
      output.writeByte(damage);
      output.writeInt(pierceAC);
      output.writeInt(bashAC);
      output.writeInt(slashAC);
      output.writeInt(magicAC);
      output.writeUTF(immFlags);
      output.writeUTF(resFlags);
      output.writeUTF(vulnFlags);
      output.writeUTF(detectFlags);
      output.writeByte(startPosition);
      output.writeByte(defaultPosition);
      output.writeByte(sex);
      output.writeInt(treasure);
      output.writeUTF(corpseFlags);
      output.writeUTF(formFlags);
      output.writeUTF(partsFlags);
      output.writeByte(size);
      output.writeUTF(material);
      output.writeUTF(special);

      output.writeBoolean(isShopKeeper);
      output.writeInt(nbItemSold);
      output.writeInt(itemSold.length);
      for(int i=0 ; i<itemSold.length ; ++i)
        output.writeInt(itemSold[i]);

      output.writeInt(profitBuy);
      output.writeInt(profitSell);
      output.writeByte(openHour);
      output.writeByte(closeHour);

      output.writeInt(specialProgs.length);
      for(int i=0 ; i<specialProgs.length ; ++i)
        output.writeUTF(specialProgs[i]);

      output.writeInt(practicer.length);
      for(int i=0 ; i<practicer.length ; ++i)
        output.writeBoolean(practicer[i]);
   }

   public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
   {
      super.readObject(input);

      //instances
      instances.clear();
      int tempSize = input.readInt();
      for(int i=0 ; i<tempSize ; ++i)
      {
         instances.add(new Long(input.readLong()));
      }

      nbInstancesPerRoom.clear();
      tempSize = input.readInt();
      for(int i=0; i<tempSize ; ++i)
      {
         nbInstancesPerRoom.put(new Long(input.readLong()), new Integer(input.readInt()));
      }

      //datas
      setVnum(input.readInt());
      nameList = input.readUTF();
      shortDescription = input.readUTF();
      longDescription = input.readUTF();
      lookDescription = input.readUTF();
      race = input.readInt();
      actFlags = input.readUTF();
      affFlags = input.readUTF();
      offFlags = input.readUTF();
      alignment = input.readInt();
      mobileGroup = input.readInt();
      level = input.readInt();
      hitBonus = input.readInt();
      hitDice.readObject(input);
      manaDice.readObject(input);
      damageDice.readObject(input);
      damageType = input.readByte();
      damage = input.readByte();
      pierceAC = input.readInt();
      bashAC = input.readInt();
      slashAC = input.readInt();
      magicAC = input.readInt();
      immFlags = input.readUTF();
      resFlags = input.readUTF();
      vulnFlags = input.readUTF();
      detectFlags = input.readUTF();
      startPosition = input.readByte();
      defaultPosition = input.readByte();
      sex = input.readByte();
      treasure = input.readInt();
      corpseFlags = input.readUTF();
      formFlags = input.readUTF();
      partsFlags = input.readUTF();
      size = input.readByte();
      material = input.readUTF();
      special = input.readUTF();

      isShopKeeper = input.readBoolean();
      nbItemSold = input.readInt();
      itemSold = new int[input.readInt()];
      for(int i=0 ; i<itemSold.length ; ++i)
        itemSold[i] = input.readInt();

      profitBuy = input.readInt();
      profitSell = input.readInt();
      openHour = input.readByte();
      closeHour = input.readByte();

     //I foresee further change of the tab size, so it is managed here
     tempSize = input.readInt();
     for(int i=0 ; i<Math.min(tempSize, AnatoliaConsts.mobileProgTexts.length) ; ++i)
       specialProgs[i] = input.readUTF();
     //in case we add some functionnalities
     if (tempSize < AnatoliaConsts.mobilePracticerTexts.length)
     {
       for(int i=tempSize ; i<AnatoliaConsts.mobileProgTexts.length ; ++i)
         specialProgs[i] = "";
     }

     //I foresee further change of the tab size, so it is managed here
     tempSize = input.readInt();
     for(int i=0 ; i<Math.min(tempSize, AnatoliaConsts.mobilePracticerTexts.length) ; ++i)
       practicer[i] = input.readBoolean();
     //in case we add some functionnalities
     if (tempSize < AnatoliaConsts.mobilePracticerTexts.length)
     {
       for(int i=tempSize ; i<AnatoliaConsts.mobilePracticerTexts.length ; ++i)
         practicer[i] = false;
     }
   }

   public void writeDescription(FileWriter output) throws IOException
   {
      if ((instances.size() == 0) || isExtern()) return;
      String tempMaterial = material;
      if (!tempMaterial.trim().equals("")) material = "'"+material+"'";
      output.write("#"+super.getVnum()+"\n");
      output.write(nameList+"~"+"\n");
      output.write(shortDescription+"~"+"\n");
      output.write(longDescription+"\n"+"~"+"\n");
      output.write(lookDescription+"\n"+"~"+"\n");
      output.write(AnatoliaConsts.getRaceTexts()[race]+"~"+"\n");
      int group = (mobileGroup>0) ? (myCollection.areaMap.areaNumber*1000 + mobileGroup) : 0;
      output.write(actFlags+" "+affFlags+" "+alignment+" "+group+"\n");
      output.write(level+" "+hitBonus+" "+hitDice+" "+manaDice+" "+damageDice+" ");
      output.write(AnatoliaConsts.damageTexts[damageType][damage]+"\n");
      output.write(pierceAC+" "+bashAC+" "+slashAC+" "+magicAC+"\n");
      output.write(offFlags+" "+immFlags+" "+resFlags+" "+vulnFlags+"\n");
      output.write(AnatoliaConsts.positionTexts[startPosition]+" "+AnatoliaConsts.positionTexts[startPosition]+" "+AnatoliaConsts.sexTexts[sex]+" "+treasure+"\n");
      output.write(formFlags+" "+partsFlags+" "+AnatoliaConsts.doorSizeTexts[size].getCode()+" "+"0"+"\n");//"'"+tempMaterial+"'"+"\n");

      //removing some flags
      char flag;
      String flagsToRemove = "";
      MobileRaceFlagsValues recommendedFlags = AnatoliaConsts.mobileRecommendedRaceFlagsValues[race];
      //act
      for(int i=0 ; i<recommendedFlags.getActFlags().length() ; ++i)
      {
         flag = recommendedFlags.getActFlags().charAt(i);
         if (actFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F act "+flagsToRemove+"\n");
      flagsToRemove = "";
      //aff
      for(int i=0 ; i<recommendedFlags.getAffFlags().length() ; ++i)
      {
         flag = recommendedFlags.getAffFlags().charAt(i);
         if (affFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F aff "+flagsToRemove+"\n");
      flagsToRemove = "";
      //off
      for(int i=0 ; i<recommendedFlags.getOffFlags().length() ; ++i)
      {
         flag = recommendedFlags.getOffFlags().charAt(i);
         if (offFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F off "+flagsToRemove+"\n");
      flagsToRemove = "";
      //imm
      for(int i=0 ; i<recommendedFlags.getImmFlags().length() ; ++i)
      {
         flag = recommendedFlags.getImmFlags().charAt(i);
         if (immFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F imm "+flagsToRemove+"\n");
      flagsToRemove = "";
      //res
      for(int i=0 ; i<recommendedFlags.getResFlags().length() ; ++i)
      {
         flag = recommendedFlags.getResFlags().charAt(i);
         if (resFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F res "+flagsToRemove+"\n");
      flagsToRemove = "";
      //vul
      for(int i=0 ; i<recommendedFlags.getVulnFlags().length() ; ++i)
      {
         flag = recommendedFlags.getVulnFlags().charAt(i);
         if (vulnFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F vul "+flagsToRemove+"\n");
      flagsToRemove = "";
      //for
      for(int i=0 ; i<recommendedFlags.getFormFlags().length() ; ++i)
      {
         flag = recommendedFlags.getFormFlags().charAt(i);
         if ((corpseFlags+formFlags).indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F for "+flagsToRemove+"\n");
      flagsToRemove = "";
      //par
      for(int i=0 ; i<recommendedFlags.getPartsFlags().length() ; ++i)
      {
         flag = recommendedFlags.getPartsFlags().charAt(i);
         if (partsFlags.indexOf(flag)<0) flagsToRemove += flag;
      }
      if (!flagsToRemove.trim().equals("")) output.write("F par "+flagsToRemove+"\n");
      flagsToRemove = "";
   }

   public void writeReset(FileWriter output) throws IOException
   {
      if (instances.size() == 0) return;
      Iterator it = instances.iterator();
      InstanceOfMobile instance = null;
      int localTotal;

      while(it.hasNext())
      {
         instance = (InstanceOfMobile) it.next();
         localTotal = ((Integer) nbInstancesPerRoom.get(AreaBuilderConsts.getRoomNode(instance.getTheParent()))).intValue();

         output.write("M 0 "+super.getVnum()+" "+instances.size()+" ");
         output.write(AreaBuilderConsts.getRoomNode(instance.getTheParent()).getVnum()+" "+localTotal);
         output.write("\t* mobile <"+nameList+">"+"\n");

         //if there is equipment, load it NOW
         InstanceOfObject equipment = null;
         for(int i=0 ; i<instance.getChildCount() ; ++i)
         {
            equipment = (InstanceOfObject) instance.getChildAt(i);
            if (equipment.equipLocation >= 0)
            {
              if (equipment.equipLocation == AnatoliaConsts.WEAR_NONE) //inventory
                 output.write("G 0 "+equipment.anatoliaObject.getVnum()+" 0");
              else // !inventory
                 output.write("E 0 "+equipment.anatoliaObject.getVnum()+" 0 "+AnatoliaConsts.wearTexts[equipment.equipLocation].getCode());

              output.write("*\t equips mobile <"+this.nameList+"> with object <"+equipment.anatoliaObject.name+"> on "+AnatoliaConsts.wearTexts[equipment.equipLocation].getName());
              output.write("\n");
            }
         }
      }
   }

  public void writeShops(FileWriter output) throws IOException
  {
     if (!isShopKeeper) return;

     output.write(""+ super.getVnum() + " ");
     for(int i=0 ; i<itemSold.length ; ++i)
     {
       output.write(itemSold[i] + " ");
     }
     output.write(""+ profitBuy + " " + profitSell + " ");
     output.write(""+Math.min(23, Math.max(0, openHour))+" ");
     output.write(""+Math.min(23, Math.max(0, closeHour))+" ");
     output.write("\t*shopkeeper <" + this.nameList + ">"+"\n");
  }

   public void writeSpecial(FileWriter output) throws IOException
   {
      if (!special.trim().equals("")) output.write("M "+super.getVnum()+" "+special+"\t* Mobile <"+nameList+">"+"\n");
   }

  public void writeProg(FileWriter output) throws IOException
  {
     String s = null;
     for(int i=0 ; i<specialProgs.length ; ++i)
     {
       s = specialProgs[i];
       if ((s != null) && (!s.trim().equals("")))
       {
         output.write("M "+getVnum()+" "+AnatoliaConsts.objectProgTexts[i]+" "+s);
         output.write("\t* mobile <"+nameList+">"+"\n");
       }
     }
   }

  public void writePracticer(FileWriter output) throws IOException
  {
     for(int i=0 ; i<practicer.length ; ++i)
     {
        if (practicer[i])
         output.write("M "+getVnum()+" "+AnatoliaConsts.mobilePracticerTexts[i]);
         output.write("\t* mobile <"+nameList+">"+"\n");
     }
  }

   public void establishLinks()
   {
     LinkedList newInstances = new LinkedList();
     Iterator it = instances.iterator();
     while(it.hasNext())
     {
        newInstances.add(LinkingAndSelectableNodeObject.lookForObjectOfID((Long) it.next()));
     }
     instances = newInstances;

     Map tempNbInstancesPerRoom = new HashMap();
     long theLongID = 0;
     Set entrySet = nbInstancesPerRoom.entrySet();
     it = entrySet.iterator();
     Map.Entry entry = null; //<roomID, nbInstances>
     while(it.hasNext())
     {
        entry = (Map.Entry) it.next();
        theLongID = ((Long) entry.getKey()).longValue();
        tempNbInstancesPerRoom.put(((Room) Room.lookForObjectOfID(theLongID)).anatoliaRoom,
                                    entry.getValue()); //<@AnatoliaRoom, nbInstances>
     }
     nbInstancesPerRoom = tempNbInstancesPerRoom;
   }

  public String toString()
  {
     return "mob<"+nameList+">;vnum "+super.getVnum();
  }
}
