/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.io.*;
import java.util.*; //pour StringTokenizer

public class AnatoliaMobilesReader extends AnatoliaAreReader
{
  public AnatoliaMobilesReader(AreaMap areaMap)
  {
    super(areaMap);
  }

  public void read(BufferedReader input) throws IOException
  {
     String line = super.trim(input);
     while(!line.startsWith("#0"))
     {
       AnatoliaMobile mobile = new AnatoliaMobile(areaMap.theMobiles);

       //vnum
       //mobile.setVnum(MyInteger.parseInt(line.substring(1)));
       mobile.imposeVnum(MyInteger.parseInt(line.substring(1)));

       //name list
       line = super.trim(input);
       while(line.indexOf("~")<0) line = line + super.trim(input);
       mobile.nameList = line.substring(0,Math.max(0,line.length()-1));

       //short desc
       line = super.trim(input);
       mobile.shortDescription = line.substring(0,Math.max(0,line.length()-1));

       //long desc
       line = "";
       do
       {
         line = line + "\n" + super.trim(input);
       } while (!line.endsWith("~"));
       mobile.longDescription = line.substring(0,Math.max(0,line.length()-1)).trim();

       //look desc
       line = "";
       do
       {
         line = line + "\n" + super.trim(input);
       } while (!line.endsWith("~"));
       mobile.lookDescription = line.substring(0,Math.max(0,line.length()-1)).trim();

       //race
       line = "";
       do
       {
         line = line + "\n"+super.trim(input).toLowerCase();
       } while (!line.endsWith("~"));
       line = line.substring(0,line.indexOf("~")).trim();
       mobile.race = Math.max(0, AreaBuilderConsts.findInTab(AnatoliaConsts.getRaceTexts(), line));

       //act aff align group
       line = super.trim(input);
       StringTokenizer tokens = new StringTokenizer(line);
       mobile.actFlags = tokens.nextToken() + AnatoliaConsts.mobileActFlagTexts[AnatoliaConsts.MOBILE_ACT_FLAG_IS_NPC].getCode();
       mobile.affFlags = tokens.nextToken();
       mobile.alignment = MyInteger.parseInt(tokens.nextToken());
       mobile.mobileGroup = MyInteger.parseInt(tokens.nextToken())%1000;

       //<level> <+to hit> <hit dice> <mana dice> <damage> <damage type>
       line = super.trim(input);
       tokens = new StringTokenizer(line);
       mobile.level = MyInteger.parseInt(tokens.nextToken());
       mobile.hitBonus = MyInteger.parseInt(tokens.nextToken());
       mobile.hitDice    = new DiceRoll(tokens.nextToken());
       mobile.manaDice   = new DiceRoll(tokens.nextToken());
       mobile.damageDice = new DiceRoll(tokens.nextToken());
       String damage = tokens.nextToken();
       mobile.damageType = (byte) Math.max(0,AnatoliaConsts.getDamageType(damage));
       mobile.damage = (byte) Math.max(0,AreaBuilderConsts.findInTab(AnatoliaConsts.damageTexts[mobile.damageType], damage));

       //AC
       line = super.trim(input);
       tokens = new StringTokenizer(line);
       mobile.pierceAC = MyInteger.parseInt(tokens.nextToken());
       mobile.bashAC = MyInteger.parseInt(tokens.nextToken());
       mobile.slashAC = MyInteger.parseInt(tokens.nextToken());
       mobile.magicAC = MyInteger.parseInt(tokens.nextToken());

       //Off imm res vuln
       line = super.trim(input);
       tokens = new StringTokenizer(line);
       mobile.offFlags = tokens.nextToken();
       mobile.immFlags = tokens.nextToken();
       mobile.resFlags = tokens.nextToken();
       mobile.vulnFlags= tokens.nextToken();

       //startpos defpos sex treas
       line = super.trim(input).toLowerCase();
       tokens = new StringTokenizer(line);
       String str_position = tokens.nextToken();
       int int_position = MyInteger.parseInt(str_position);
       if (int_position <= 0)
       {
         int_position = AreaBuilderConsts.findInTab(AnatoliaConsts.positionTexts,str_position);
         if (int_position<0) int_position = 8;
       }
       mobile.startPosition   = (byte) int_position;
       if (mobile.startPosition < AnatoliaConsts.POSITION_SLEEPING)
         mobile.startPosition = AnatoliaConsts.POSITION_STANDING;

       str_position = tokens.nextToken();
       int_position = MyInteger.parseInt(str_position);
       if (int_position <= 0)
       {
         int_position = AreaBuilderConsts.findInTab(AnatoliaConsts.positionTexts,str_position);
         if (int_position<0) int_position = 8;
       }
       mobile.defaultPosition   = (byte) int_position;
       if (mobile.defaultPosition < AnatoliaConsts.POSITION_SLEEPING)
         mobile.defaultPosition = AnatoliaConsts.POSITION_STANDING;


       String str_sex = tokens.nextToken();
       int    int_sex = MyInteger.parseInt(str_sex);
       if ((int_sex <= 0) && (!str_sex.trim().startsWith("0")))
       {
         int_sex = AreaBuilderConsts.findInTab(AnatoliaConsts.sexTexts,str_sex);
         if (int_sex<0) int_sex = 0;
       }
       mobile.sex   = (byte) int_sex;
       mobile.treasure        = MyInteger.parseInt(tokens.nextToken());

       //Form part size material
       line = super.trim(input);
       tokens = new StringTokenizer(line);
       mobile.formFlags  = tokens.nextToken();
       mobile.partsFlags = tokens.nextToken();
       String str_size = tokens.nextToken();
       int    int_size = MyInteger.parseInt(str_size);
       if ((int_size <= 0) && (!str_size.trim().startsWith("0")))
       {
         int_size = AreaBuilderConsts.findInTab(AnatoliaConsts.getMobileSizeCodes(),str_size);
         if (int_size<0) int_size = 0;
       }
       mobile.size       = (byte) int_size;
       mobile.material   = "";
       if (tokens.hasMoreTokens())
         mobile.material = tokens.nextToken();
       if (mobile.material.startsWith("'"))
         mobile.material = mobile.material.substring(1);
       if (mobile.material.endsWith("'"))
         mobile.material = mobile.material.substring(0, Math.max(0, mobile.material.length()-1));

       //if some flags were remove
       line = super.trim(input);
       String group = null;
       String flags = null;
       while(line.startsWith("F ") || line.startsWith("F\t"))
       {
          try
          {
            tokens = new StringTokenizer(line);
            tokens.nextToken(); //one time for nothing : "F "
            group = tokens.nextToken(); //group = <act> or <aff>,<off>,<res>,<vul>,<imm>,<for>,<par>
            flags = tokens.nextToken();
            if (group.equals("act"))
               mobile.actFlags = super.removeElementsFromString(flags, mobile.actFlags);
            else if (group.equals("aff"))
               mobile.affFlags = super.removeElementsFromString(flags, mobile.affFlags);
            else if (group.equals("off"))
               mobile.offFlags = super.removeElementsFromString(flags, mobile.offFlags);
            else if (group.equals("res"))
               mobile.resFlags = super.removeElementsFromString(flags, mobile.resFlags);
            else if (group.equals("vul"))
               mobile.vulnFlags = super.removeElementsFromString(flags, mobile.vulnFlags);
            else if (group.equals("imm"))
               mobile.immFlags = super.removeElementsFromString(flags, mobile.immFlags);
            else if (group.equals("for"))
               mobile.formFlags = super.removeElementsFromString(flags, mobile.formFlags);
            else if (group.equals("par"))
               mobile.partsFlags = super.removeElementsFromString(flags, mobile.partsFlags);
          }
          catch(NoSuchElementException e) {}
          line = super.trim(input);
        } //end while Flags removed
        areaMap.theMobiles.add(mobile);
     } //end while mob
  } //end function
} //end class