/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*; //for LinkedList
import java.io.*;   //for archivation
import javax.swing.tree.*;

public class AnatoliaRoom extends AnatoliaThing implements Serializable
{
   String roomName      = "A room";
   String roomDescription = "This room is boring";
   LinkedList extras = new LinkedList(); // Pair<String keywords, String description>
   boolean[] flags = new boolean[AnatoliaConsts.roomFlagsTexts.length];
   byte randomUpTo = AnatoliaConsts.NORTH;
   byte sectorType = 0;
   int HPRecovery = 100;
   int MPRecovery = 100;
   DoorExit[] doors = new DoorExit[6];

   Room myRoom;

   public AnatoliaRoom(Room myRoom, AnatoliaCollection anatoliaCollection)
   {
      super(anatoliaCollection);
      this.myRoom = myRoom;
   }

   public AnatoliaRoom pseudoClone(Room room)
   {
      AnatoliaRoom anatoliaRoom = new AnatoliaRoom(room, super.myCollection);
      //DO NOT anatoliaRoom.vnum = this.vnum;//
      anatoliaRoom.roomName = new String(this.roomName);
      anatoliaRoom.roomDescription = new String(this.roomDescription);
      anatoliaRoom.extras = (LinkedList) this.extras.clone();

      for(int i=0 ; i<flags.length ; ++i)
         anatoliaRoom.flags[i] = this.flags[i];
      anatoliaRoom.sectorType = this.sectorType;
      anatoliaRoom.HPRecovery = this.HPRecovery;
      anatoliaRoom.MPRecovery = this.MPRecovery;
      //DO NOT
      //for(int i=0 ; i<6 ; ++i)
      //   anatoliaRoom.doors[i] = (DoorExit) this.doors[i].clone();

      return anatoliaRoom;
   }

   public void writeObject(ObjectOutputStream output) throws IOException
   {
      super.writeObject(output);
      output.writeLong(this.getObjectID());
      //output.writeInt(super.getVnum());
      output.writeUTF(roomName);
      output.writeUTF(roomDescription);
      output.writeInt(extras.size()); //<String keywords, String description>
      String keywords = null;
      String desc = null;
      Iterator it = extras.listIterator();
      Pair pair = null;
      while (it.hasNext())
      {
         pair = (Pair) it.next();
         keywords = (String) pair.getFirst();
         desc = (String) pair.getSecond();
         output.writeUTF(keywords);
         output.writeUTF(desc);
      }

      output.writeInt(flags.length);
      for(int i=0 ; i<flags.length ; ++i)
         output.writeBoolean(flags[i]);

      output.writeByte(sectorType);
      output.writeInt(HPRecovery);
      output.writeInt(MPRecovery);
      DoorExit door = null;
      for(int i=0 ; i<6 ; ++i)
      {
         door = doors[i];
         if (door == null)
            output.writeBoolean(false);
         else
         {
            output.writeBoolean(true);
            door.writeObject(output);
         }
      }
   }

   public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
   {
      super.readObject(input);
      this.setObjectID(input.readLong());
      //super.setVnum(input.readInt());
      roomName = input.readUTF();
      roomDescription = input.readUTF();
      int tempSize = input.readInt();
      String keywords = null;
      String desc = null;
      for(int i=0 ; i<tempSize ; ++i)
      {
         keywords = (String) input.readUTF();
         desc = (String) input.readUTF();
         extras.add(new Pair(keywords, desc));
      }

      //I foresee further tab change size
      tempSize = input.readInt();
      for(int i=0 ; i<Math.min(tempSize, flags.length) ; ++i)
         flags[i] = input.readBoolean();
      //In cas we add some functionnalities, it is managed here
      if (tempSize < flags.length)
      {
        for(int i=tempSize ; i<flags.length ; ++i)
           flags[i] = false;
      }

      sectorType = input.readByte();
      HPRecovery = input.readInt();
      MPRecovery = input.readInt();
      DoorExit door = new DoorExit();
      for(int i=0 ; i<6 ; ++i)
      {
         if (input.readBoolean())
         {
            door.readObject(input);
            doors[i] = (DoorExit) door.clone();
         }
         else
            doors[i] = null;
      }
   }

   public void writeDescription(FileWriter output) throws IOException
   {
      if (this.isExtern()) return;

      output.write("#"+super.getVnum()+"\n");
      output.write(this.roomName+"~\n");
      output.write(this.roomDescription+"\n");
      output.write("~\n");
      String tempString = "";
      for(int i=0 ; i<this.flags.length ; ++i)
        if (this.flags[i]) tempString += AnatoliaConsts.roomFlagsTexts[i].getCode();
      if (tempString.trim().equals("")) tempString = "0";
      output.write("0 "+tempString + " " + this.sectorType + "\n");
      for(int i=0 ; i<this.doors.length ; ++i)
      {
         if (doors[i] != null)
         {
            output.write("D"+i+"\n");
            if (!doors[i].extraDescription.trim().equals(""))
              output.write(doors[i].extraDescription+"\n");
            output.write("~\n");
            if (doors[i].doorKeyword != null ) output.write(doors[i].doorKeyword);
            output.write("~"+"\n");
            output.write(doors[i].doorState+" ");
            AnatoliaObject key = (AnatoliaObject) super.myCollection.areaMap.theObjects.getAnatoliaThingOfVnum(doors[i].keyVnum);
            if ( (key == null) || (key.type != AnatoliaConsts.OBJECT_TYPE_KEY))
               doors[i].keyVnum = 0;
            output.write(doors[i].keyVnum+" ");
            output.write(myRoom.getConnectingToRoom((byte) i).anatoliaRoom.getVnum()+" ");
            output.write(doors[i].doorSize+"\n");
         }
      }
      Iterator it = this.extras.iterator();
      Pair pair = null; //Pair<String keywords, String description>
      while(it.hasNext())
      {
         pair  = (Pair) it.next();
         output.write("E\n");
         output.write((String) pair.getFirst()+"~\n");
         output.write((String) pair.getSecond()+"\n");
         output.write("~\n");
      }
      output.write("M "+this.MPRecovery+ " H "+this.HPRecovery+"\n");
      output.write("S\n");
   }

   public void writeReset(FileWriter output) throws IOException
   {
      if (this.flags[AnatoliaConsts.ROOM_FLAG_RANDOM])
      {
         output.write("R 0 "+super.getVnum()+" "+this.randomUpTo+"\t* Randomize exits of room <"+this.roomName+">"+"\n");
      }
      else
      {
         for(int i=0 ; i<6 ; ++i)
         {
            if (this.doors[i]!= null)
            {
               if (this.doors[i].doorType != AnatoliaConsts.EXIT_WITHOUT_DOOR)
               {
                 output.write("D 0 "+super.getVnum()+" "+i+" "+this.doors[i].doorState+"\t* door "+AnatoliaConsts.directionTexts[i]+" of room <"+roomName+">");
                 //doors[i].writeReset(output); //lock flags
                 output.write("\n");
               }
            }
         }
      }
   }

   public void establishLinks()
   {
   }

   public String toString()
   {
      return new String("room<"+this.roomName+">;vnum "+super.getVnum());
   }

}

class DoorExit implements Serializable
{
   boolean doorType = AnatoliaConsts.EXIT_WITHOUT_DOOR;
   boolean autoSetSymetricDoor = false;
   boolean[] doorResetFlags = new boolean[AnatoliaConsts.doorResetsTexts.length];

   String doorKeyword = null;
   byte doorState = AnatoliaConsts.DOOR_STATE_OPENED;
   int keyVnum;
   byte doorSize = AnatoliaConsts.SIZE_GARGANTUAN;
   String extraDescription = "";

   public Object clone()
   {
      DoorExit doorExit = new DoorExit();
      doorExit.doorType = this.doorType;
      doorExit.autoSetSymetricDoor = this.autoSetSymetricDoor;
      for(int i=0; i<doorResetFlags.length ; ++i) doorExit.doorResetFlags[i] = this.doorResetFlags[i];
      doorExit.doorState = this.doorState;
      doorExit.keyVnum = this.keyVnum;
      doorExit.doorSize = this.doorSize;
      doorExit.extraDescription = new String(this.extraDescription);
      return doorExit;
   }

   public void writeObject(ObjectOutputStream output) throws IOException
   {
     output.writeBoolean(doorType);
     output.writeBoolean(autoSetSymetricDoor);
     if (doorKeyword != null)
     {
        output.writeBoolean(true);
        output.writeUTF(doorKeyword);
     }
     else
        output.writeBoolean(false);
     output.writeInt(doorResetFlags.length);
     for(int i=0 ; i<doorResetFlags.length ; output.writeBoolean(doorResetFlags[i++]));
     output.writeByte(doorState);
     output.writeInt(keyVnum);
     output.writeByte(doorSize);
     output.writeUTF(extraDescription);
  }

  public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
  {
     doorType = input.readBoolean();
     autoSetSymetricDoor = input.readBoolean();
     if (input.readBoolean()) doorKeyword  = input.readUTF();
     int tempSize = input.readInt();
     for(int i=0 ; i<tempSize ; doorResetFlags[i++] = input.readBoolean());
     doorState = input.readByte();
     keyVnum = input.readInt();
     doorSize = input.readByte();
     extraDescription = input.readUTF();
  }

  public void writeReset(FileWriter output) throws IOException
  {
     String tempString = "";
     for(int i=0; i<this.doorResetFlags.length ; ++i)
     {
        if (doorResetFlags[i]) tempString += AnatoliaConsts.doorResetsTexts[i].getCode();
     }
     if (tempString.trim().equals("")) tempString = "0";
     output.write(tempString+'\t'+"* door "+"\n");
  }
}