/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class FindReplaceStringDialog extends JDialog implements ActionListener
{
  AreaMap areaMap = null;
  JTextField vnumTextField = new JTextField(5);
  JTextField textToFindTextField = new JTextField(20);
  JCheckBox matchUpCaseCheckBox = new JCheckBox("Match up case", false);
  JButton findButton = new JButton("Find");
  JButton cancelButton = new JButton("Annuler");

  public FindReplaceStringDialog(AreaMap areaMap, String title, boolean modal)
  {
    super((JFrame) null, title, modal);
    this.areaMap = areaMap;

    Container contentPane = getContentPane();
    contentPane.setLayout(new GridBagLayout());
    GridBagConstraints constraint = new GridBagConstraints();

    JPanel dataPanel = new JPanel(new GridLayout(3,1));
    dataPanel.add(new JLabel("Text to Find"));
    dataPanel.add(textToFindTextField);
    dataPanel.add(matchUpCaseCheckBox);
    constraint.gridx=0;constraint.gridy=0;
    constraint.gridwidth=2;constraint.gridheight=1;
    constraint.weightx=100;constraint.weighty=40;
    contentPane.add(dataPanel, constraint);

    JPanel vnumPanel = new JPanel(new GridLayout(2,1));
    vnumPanel.add(new JLabel("search only with vnum"));
    vnumPanel.add(vnumTextField);
    constraint.gridx=0;constraint.gridy=1;
    constraint.gridwidth=2;constraint.gridheight=1;
    constraint.weightx=100;constraint.weighty=20;
    contentPane.add(vnumPanel, constraint);

    constraint.gridx=0;constraint.gridy=2;
    constraint.gridwidth=1;constraint.gridheight=1;
    constraint.weightx=50;constraint.weighty=20;
    contentPane.add(findButton, constraint);

    constraint.gridx=1;constraint.gridy=2;
    constraint.gridwidth=1;constraint.gridheight=1;
    constraint.weightx=50;constraint.weighty=20;
    contentPane.add(cancelButton, constraint);

    findButton.addActionListener(this);
    cancelButton.addActionListener(this);

    pack();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension mySize = this.getSize();
    this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                     (int) (screenSize.getHeight()-mySize.getHeight())/2
                    );
  }

  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() == cancelButton)
    {
      setVisible(false);
    }
    else if (e.getSource() == findButton)
    {
      performFind();
    }
  }

  public void performFind()
  {
    //vnum == -1 if not precised
    int vnum = -1;
    if (!vnumTextField.getText().trim().equals(""))
       vnum = MyInteger.parseInt(vnumTextField.getText());

    boolean matchUpCase = matchUpCaseCheckBox.isSelected();
    String textToFind = textToFindTextField.getText();
    FindStringResultDialog results = new FindStringResultDialog(this, "Results", false);
    Set setOfMatchingRooms = new HashSet();
    Set setOfMatchingMobiles = new HashSet();
    Set setOfMatchingObjects = new HashSet();
    Iterator it = areaMap.theRooms.getIterator();
    Iterator it2 = null;
    Pair pair = null;
    int j;
    int found;
    while(it.hasNext())
    {
       AnatoliaRoom room = ((Room) it.next()).anatoliaRoom;
       if ((vnum>=0) && (room.getVnum() != vnum)) continue; //filter the vnum

       if (matchUpCase) found = room.roomName.indexOf(textToFind);
       else found = room.roomName.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingRooms.add(room);

       if (matchUpCase) found = room.roomDescription.indexOf(textToFind);
       else found = room.roomDescription.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingRooms.add(room);

       it2 = room.extras.iterator();
       j=0;
       while(it2.hasNext())
       {
         pair = (Pair) it2.next(); // Pair<String keywords, String description>
         if (matchUpCase) found = ((String) pair.getFirst()).indexOf(textToFind);
         else found = ((String) pair.getFirst()).toLowerCase().indexOf(textToFind.toLowerCase());
         if (found >= 0) setOfMatchingRooms.add(room);

         if (matchUpCase) found = ((String) pair.getSecond()).indexOf(textToFind);
         else found = ((String) pair.getSecond()).toLowerCase().indexOf(textToFind.toLowerCase());
         if (found >= 0) setOfMatchingRooms.add(room);

         ++j;
       }
    }

    it = areaMap.theMobiles.getIterator();
    while(it.hasNext())
    {
       AnatoliaMobile mobile = (AnatoliaMobile) it.next();
       if ((vnum>=0) && (mobile.getVnum() != vnum)) continue; //filter the vnum

       if (matchUpCase) found = mobile.nameList.indexOf(textToFind);
       else found = mobile.nameList.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingMobiles.add(mobile);

       if (matchUpCase) found = mobile.shortDescription.indexOf(textToFind);
       else found = mobile.shortDescription.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingMobiles.add(mobile);

       if (matchUpCase) found = mobile.longDescription.indexOf(textToFind);
       else found = mobile.longDescription.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingMobiles.add(mobile);

       if (matchUpCase) found = mobile.lookDescription.indexOf(textToFind);
       else found = mobile.lookDescription.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingMobiles.add(mobile);

       if (matchUpCase) found = mobile.material.indexOf(textToFind);
       else found = mobile.material.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingMobiles.add(mobile);

       if (matchUpCase) found = mobile.special.indexOf(textToFind);
       else found = mobile.special.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingMobiles.add(mobile);
    }

    it = areaMap.theObjects.getIterator();
    while(it.hasNext())
    {
       AnatoliaObject object = (AnatoliaObject) it.next();
       if ((vnum>=0) && (object.getVnum() != vnum)) continue; //filter the vnum

       if (matchUpCase) found = object.name.indexOf(textToFind);
       else found = object.name.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingObjects.add(object);

       if (matchUpCase) found = object.shortDescription.indexOf(textToFind);
       else found = object.shortDescription.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingObjects.add(object);

       if (matchUpCase) found = object.longDescription.indexOf(textToFind);
       else found = object.longDescription.toLowerCase().indexOf(textToFind.toLowerCase());
       if (found >= 0) setOfMatchingObjects.add(object);

       it2 = object.extraDescriptions.iterator();
       j=0;
       while(it2.hasNext())
       {
         pair = (Pair) it2.next(); //Pair<keyword, description>
         if (matchUpCase) found = ((String) pair.getFirst()).indexOf(textToFind);
         else found = ((String) pair.getFirst()).toLowerCase().indexOf(textToFind.toLowerCase());
         if (found >= 0) setOfMatchingObjects.add(object);

         if (matchUpCase) found = ((String) pair.getSecond()).indexOf(textToFind);
         else found = ((String) pair.getSecond()).toLowerCase().indexOf(textToFind.toLowerCase());
         if (found >= 0) setOfMatchingObjects.add(object);

         ++j;
       }
    }

    LinkedList listOfStringLocations = new LinkedList();
    it = setOfMatchingRooms.iterator();
    while(it.hasNext())
       listOfStringLocations.add(new StringLocation(areaMap, (AnatoliaThing) it.next()));
    it = setOfMatchingMobiles.iterator();
    while(it.hasNext())
       listOfStringLocations.add(new StringLocation(areaMap, (AnatoliaThing) it.next()));
    it = setOfMatchingObjects.iterator();
    while(it.hasNext())
       listOfStringLocations.add(new StringLocation(areaMap, (AnatoliaThing) it.next()));

    int size = listOfStringLocations.size();
    if (size == 0)
       JOptionPane.showMessageDialog(null, "No occurence found", "Find failed", JOptionPane.INFORMATION_MESSAGE);
    else
      results.showFor(listOfStringLocations);

    areaMap.repaint();
  }
}