/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*; //Iterator

public class LayersDialog extends JDialog implements WindowListener, ActionListener
{
  AreaMap areaMap;
  JScrollPane scrollPane = null;
  JCheckBox[] layerCheckBoxes = new JCheckBox[0];
  int[] layerValues = new int[0];
  JPanel panel = new JPanel();

  public LayersDialog(Frame owner, String title, boolean modal, AreaMap areaMap)
  {
     super(owner, title, modal);

     this.areaMap = areaMap;
     scrollPane = new JScrollPane(panel,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
     for(int i=0 ; i<layerCheckBoxes.length ; ++i)
     {
       panel.add(layerCheckBoxes[i]);
     }
     getContentPane().add(scrollPane, BorderLayout.CENTER);
     addWindowListener(this);
     pack();
     Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
     Dimension mySize = this.getSize();
     this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                      (int) (screenSize.getHeight()-mySize.getHeight())/2
                     );
  }

  public void update()
  {
    SortedSet layers = new TreeSet();
    Set roomsToExplore = new HashSet();
    Iterator itRooms = areaMap.theRooms.getIterator();
    while(itRooms.hasNext()) {roomsToExplore.add(itRooms.next());}

    while(!roomsToExplore.isEmpty())
    {
      Room rootRoom = (Room) roomsToExplore.iterator().next();
      roomsToExplore.remove(rootRoom);
      LinkedList neighbours = new LinkedList();
      neighbours.addFirst(rootRoom);
      while(!neighbours.isEmpty())
      {
        Room currentRoom = (Room) neighbours.removeFirst();
        layers.add(new Integer(currentRoom.layer));
        for(int i=0 ; i<6 ; ++i)
        {
          if (currentRoom.connectingToRooms[i] != null)
          {
            Room next = (Room) currentRoom.connectingToRooms[i].getFirst();
            if (roomsToExplore.contains(next))
            {
              roomsToExplore.remove(next);
              next.layer = currentRoom.layer;
              if (i == Room.DOORUP) ++next.layer;
              else if (i == Room.DOORDOWN) --next.layer;
              layers.add(new Integer(next.layer));
              neighbours.addLast(next);
            } //end if must be set
          } //end if neighbour
        } //end for each door
      } //end while neighbours
    } //end while room

    panel.removeAll();
    panel.setLayout(new GridLayout(layers.size(),1));
    layerCheckBoxes = new JCheckBox[layers.size()];
    layerValues = new int[layers.size()];
    Iterator it = layers.iterator();
    int i=0;
    int size = layerCheckBoxes.length;
    while(it.hasNext())
    {
      Integer layer = (Integer) it.next();
      layerValues[i] = layer.intValue();
      layerCheckBoxes[i] = new JCheckBox("Layer "+layer.toString(),
                                         areaMap.selectedLayers.contains(layer));
      layerCheckBoxes[i].addActionListener(this);
      ++i;
    }
    size = layerCheckBoxes.length;
    for(int k=0 ; k<size ; ++k)
    {
      panel.add(layerCheckBoxes[size-1-k]);
    }

    actionPerformed(null);

    pack();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension mySize = this.getSize();
    this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                     (int) (screenSize.getHeight()-mySize.getHeight())/2
                    );
    System.gc();
  }

  public void windowClosing(WindowEvent e)
  {
     areaMap.mainScreen.menuDisplayLayers.setSelected(false);
     actionPerformed(null);
  }
  public void windowClosed(WindowEvent e)
  {
     actionPerformed(null);
  }
  public void windowOpened(WindowEvent e)
  {
    update();
    actionPerformed(null);
  }
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e)
  {
    update();
    actionPerformed(null);
  }
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}

  public void actionPerformed(ActionEvent e)
  {
     areaMap.selectedLayers.clear();
     for(int i=0 ; i<layerValues.length ; ++i)
     {
        if (layerCheckBoxes[i].isSelected())
        {
          areaMap.selectedLayers.add(new Integer(layerValues[i]));
        }
     }
     areaMap.repaint();
  }

  public void enableAllLayers()
  {
     areaMap.selectedLayers.clear();
     for(int i=0 ; i<layerValues.length ; ++i)
     {
        layerCheckBoxes[i].setSelected(true);
        areaMap.selectedLayers.add(new Integer(layerValues[i]));
     }
  }
}