/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.util.*; //for LinkedList
import java.io.*; //for Archivation

public class ObjectCollection extends AnatoliaCollection
{
  public ObjectCollection(AreaMap areaMap)
  {
     super(areaMap);
  }

  public void buildObjectDescriptions(FileWriter output) throws IOException
  {
      Iterator it = super.collection.iterator();
      while(it.hasNext())
      {
         ((AnatoliaObject) it.next()).writeDescription(output);
      }
  }
  public void buildObjectResets(FileWriter output) throws IOException
  {
      Iterator it = super.collection.iterator();
      while(it.hasNext())
      {
         ((AnatoliaObject) it.next()).writeReset(output);
      }
  }

  public void buildObjectProgs(FileWriter output) throws IOException
  {
      Iterator it = super.collection.iterator();
      while(it.hasNext())
      {
         ((AnatoliaObject) it.next()).writeProg(output);
      }
  }

  public void buildObjectLimits(FileWriter output) throws IOException
  {
      Iterator it = super.collection.iterator();
      while(it.hasNext())
      {
         ((AnatoliaObject) it.next()).writeLimit(output);
      }
  }

  public void writeObject(ObjectOutputStream output) throws IOException
  {
      super.writeObject(output);
      output.writeInt(super.collection.size());
      Iterator it = super.collection.iterator();
      while(it.hasNext())
      {
         ((AnatoliaObject) it.next()).writeObject(output);
      }
  }

   public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
   {
      this.clear();

      super.readObject(input);
      int sizeOfObjectList = input.readInt();
      AnatoliaObject object = null;
      for(int i=0; i<sizeOfObjectList; ++i)
      {
         object = new AnatoliaObject(this);
         object.readObject(input);
         add(object);
      }
   }

   public void establishLinks()
   {
      Iterator it = super.collection.iterator();
      AnatoliaObject object = null;
      while (it.hasNext())
      {
        object = (AnatoliaObject) it.next();
        object.establishLinks();
      }
  }
}